/* 
 * CSE142, 98 Autumn
 * Homework 4
 * Rain Gods and Goddesses
 *
 * Adam Williams
 * 8336416
 * Section BF

*/

#include "gp142.h"
#include <stdio.h>
#include <math.h>


#define TRUE    		1
#define FALSE   		0

/* Configure the appearance of the buttons on the menu bar */
/* number of pixels of button height and width */
#define TOP_BUTTON_MARGIN   0
#define BUTTON_HEIGHT  	    35
#define BUTTON_WIDTH        80
#define NUM_BUTTONS 	      8

/* color of letter on buttons */
#define SELECTED_COLOR   RED
#define UNSELECTED_COLOR YELLOW

/* This is the total number of drops possible at one time. */
#define MAX_PRECIP_EVENTS	500

/* y-ccordinate of the top of the "dirt" in the landscape */
#define TOP_OF_DIRT (- 2 * (GP142_YMAX / 5))

/* For many of the GP142 functions, a line width of */
/* zero means to fill the shape being drawn. */
#define FILL 0

/* Types of precipitations we support */
#define NONE    0
#define RAIN    1
#define SNOW    2
#define FLOWER  3
#define CAT     4
#define DOG     5

/* Possible places for mouse clicks */
#define BUTTON 1
/* The above means you clicked on SOME button (doesn't say which one yet) */
#define DIRT 2
/* The above means you clicked in the dirt area */
#define WIND_BUTTON 3
/* The above means you clicked on the wind speed buttons */

/*******************  STRUCTURES ***********************************/

typedef struct {
  int   precip_type;
  int   x,y;
  int   color;
  int   age;
} precip_struc;


/*******************  FUNCTION PROTOTYPES **************************/

/* redraw all the buttons. */
/* The parameters tell which button should be highlighted */
void DrawButtons(int current_precip, int current_color);
void DrawWindSpeedSelection(int current_wind_speed);

/* Clear all percipitation, so the landscape is just as original */
void ClearAllPrecipitation(int *precip_type);


/* render (draw) all the precipitation */
void DrawAllPrecipitation(precip_struc all_precips[], int precip_count);

/* increment x and y, check for precip off screen */
void process_precipitation(precip_struc all_precips[], int *precip_count, int current_wind_speed);

/* deletes precip from the array, that is off the screen */
void clean_up_precip_array(precip_struc all_precips[], int *precip_count);

/* draw the landscape and fish in the river */
void DrawLandscape();
void DrawFish(int fish_x_start);

/* draw one precip item */
void DrawFlower(int flower_age, int x, int y);
void DrawRainDrop (int x, int y, int color);
void DrawSnowDrop (int x, int y, int color);
void DrawCat(int x, int y, int color);
void DrawDog(int x, int y, int color);

/* Return an integer which tells what KIND of click there was */
int  ClassifyMouseClick(int mouse_x, int mouse_y);

/* Given that a button was click, call an appropriate function to handle it */
void handle_button(int x, int y, int *current_precip, int *current_color,
		   int *replant);
void handle_wind_button(int *current_wind_speed, int x);


/*****************************************************************************

     MAIN

*****************************************************************************/

/* main() has two chief purposes: to contain the important variables and */
/* intialize them, and to contain the "event loop", which */
/* constantly looks to see if an event has happened, and then */
/* processes the event by calling an appropriate function. */

int main(void)
{
/* variables that store info about the precip */
  precip_struc  all_precips[MAX_PRECIP_EVENTS];
  int           precip_count = NONE; 

/* variables that store info about the most current event */
  int           quit = FALSE;
  int           mouse_x, mouse_y;
  char          key_hit;
  int           mouse_click;
  int           event;  /* the most recent GP142 event */
  int           clear_flag = FALSE;  /* true if weather is to be cleared */

/* variables that store info about the buttons currently selected */
  int           current_wind_speed = 0;
  int           current_precip = NONE, current_color = NONE;

/* variables that store info about the background */
  int           fish_x_location = 0;
  int           sun_x =GP142_XMAX - 50, sun_y = GP142_YMAX - 35;
  int           is_day = TRUE;


  GP142_open();           /* Open and init the GP142 Graphics Window      */
  GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */

  GP142_clear();	        /* This will clear the window & paint it white. */
  GP142_animate(ANI_RUN); /* Start animation so precips will grow         */

  DrawLandscape();        /* Draw the landscape                           */
  DrawFish(fish_x_location); /* Draw the fish in the river               */
 
  DrawButtons(current_precip, current_color);	  /* Show the buttons...    */ 
  DrawWindSpeedSelection(current_wind_speed);

/*
 * The main event loop:
 * --- ---- ----- -----
 * wait for the next user action, such as a mouse click or keyboard
 * keypress, or a periodic update event (allowing us to display the
 * next frame of an animation -- not used int this program)
 * and call the appropriate function to
 * handle it.  Repeat this until the user signals "quit".
 */

  while (!quit) {  /*this is the famous "event loop" */

	/*find out what has happened */
	event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);

	switch (event) {
    case GP142_QUIT:
      quit = TRUE;           /* set flag to terminate loop           */
      break;

    case GP142_KBD:
		/*not needed for this program.*/
		/*Use this if user will need to hit a key.
		/*Note that scanf CANNOT be used with GP142*/
      break;

    case GP142_MOUSE:
    /* a mouse click has occurred. */
    /* check for button versus dirt */
    /* handle appropriate button click, if any */
      mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
	    if (mouse_click == BUTTON) 
      {
			/* a button has been clicked.  Call a function to handle it*/
		    handle_button(mouse_x, mouse_y, 
		                  &current_precip, &current_color, &clear_flag);
		  if (clear_flag)
			  ClearAllPrecipitation(&precip_count);
	    }

      else if (mouse_click == WIND_BUTTON)
      {
        handle_wind_button(&current_wind_speed, mouse_x);
      }
      else if ((mouse_click != DIRT)
    		&& (current_precip != NONE) && (current_color != NONE) 
        && (precip_count < MAX_PRECIP_EVENTS))
        {
          all_precips[precip_count].precip_type  = current_precip;
          all_precips[precip_count].color = current_color;
          all_precips[precip_count].x = mouse_x;
          all_precips[precip_count].y = mouse_y;
          precip_count++;
	      }
      break;

    case GP142_PERIODIC:
    /* Some time has passed, 
   	/* so make all the precipitation a little lower */

		/* and change the sun's position a little bit */
		  sun_x --;
		  if (sun_x < -GP142_XMAX -30)  /* if sun leaves screen switch to moon */
      {                             /* or from moon to sun                 */
		  	sun_x = - sun_x;
			  sun_y = sun_y -1;
        is_day= !is_day;
				/* over a long period of time, the sun will set.*/
	  	}
      /* move precipitation */    
      process_precipitation(all_precips, &precip_count, current_wind_speed);

      fish_x_location++;  /* move fish up stream */
      if (fish_x_location == 50)
        fish_x_location = 0;
      break;

    default:
      break;
	
    }  /*end switch */

    /* Redraw everything */
    DrawLandscape(sun_x, sun_y, is_day);
    DrawFish(fish_x_location);
    DrawButtons(current_precip, current_color);
    DrawWindSpeedSelection(current_wind_speed);
    DrawAllPrecipitation(all_precips, precip_count);
 
  }  /* end event loop */


/*
 * WRAP-UP
 */
  GP142_close();         /* clean up and close graphics window   */

  return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/


/*reinitialize so that ALL current precipitation is erased. */
void  ClearAllPrecipitation(int * precip_count)
{
  *precip_count = 0;
}



#define RAINDROP_WIDTH 3
void DrawRainDrop (int x, int y, int color) { 
  GP142_circleXY(color, x, y, RAINDROP_WIDTH);
}


#define SNOWDROP_WIDTH 4
#define SNOWDROP_HEIGHT 5
void DrawSnowDrop (int x, int y, int color) {
  GP142_rectangleXY(color, x, y, x+SNOWDROP_WIDTH, y+SNOWDROP_HEIGHT,FILL);
}


void DrawCat (int x, int y, int color)
{
  int leg_move;

  GP142_ovalXY(color, x, y, x+20, y-10, FILL);
  GP142_circleXY(color, x-2, y+2, 5);
  GP142_triangleXY(color, x-7, y+9, x-8, y+4, x-4, y+5, FILL);  
  GP142_triangleXY(color, x+3, y+9, x+4, y+4, x, y+5, FILL); 
  GP142_lineXY(BLACK,x-5, y+3, x-11, y+4, 1);
  GP142_lineXY(BLACK,x-5, y+1, x-11, y, 1);
  GP142_lineXY(BLACK,x+1, y+3, x+7, y+4, 1);
  GP142_lineXY(BLACK,x+1, y+1, x+7, y, 1);
  GP142_lineXY(color, x+18, y-4, x+28, y+10, 2);
  leg_move = x % 2 * 2;
  GP142_lineXY(color, x+17, y-6, x+17, y-20+leg_move, 2);
  GP142_lineXY(color, x+3, y-6, x+3, y-20+leg_move, 2);
}


void DrawDog(int x, int y, int color)
{
  int leg_move;

  GP142_ovalXY(color, x, y, x-17, y-12, FILL);
  GP142_circleXY(color, x+2 , y+3, 5);
  GP142_ovalXY(BLACK, x, y+7, x-4, y-7, 1);
  GP142_circleXY(color, x+8, y+3, 2);
  GP142_circleXY(BLACK, x+10,y+3,1);
  GP142_lineXY(color, x-17,y-6,x-27,y-11,2);
  leg_move = x % 2 * 2;
  GP142_lineXY(color, x-15, y-6, x-15, y-20+leg_move, 2);
  GP142_lineXY(color, x-3, y-8, x-3, y-20+leg_move, 2);
}

void DrawAllPrecipitation(precip_struc all_precips[], int precip_count)
{
int   i;
  for (i = 0; i < precip_count; i++)
  {
  	switch (all_precips[i].precip_type) 
    {
		  case RAIN   :
			  DrawRainDrop (all_precips[i].x, all_precips[i].y,all_precips[i]. color);
  			break;
	  	case SNOW   :
		  	DrawSnowDrop (all_precips[i].x, all_precips[i].y, all_precips[i].color);
			  break;
      case FLOWER :
        DrawFlower  (all_precips[i].age, all_precips[i].x, all_precips[i].y);
        break;
      case CAT:
        DrawCat(all_precips[i].x, all_precips[i].y, all_precips[i].color);
        break;
      case DOG:
        DrawDog(all_precips[i].x, all_precips[i].y, all_precips[i].color);
		  default:
			/*Bad news to ever get here.*/
			{}

		} /* end switch*/
		
	} /* end for   */
}


#define NUM_W_BUT     11        /* number of wind buttons */
#define W_BUT_WIDTH   20    
#define W_BUT_HEIGHT  20
#define W_X_START     0- NUM_W_BUT * W_BUT_WIDTH / 2
#define W_Y_START     -GP142_YMAX + 60
void DrawWindSpeedSelection(int current_wind_speed)
{
  int i;
  int color_w_speed;

  /* draw the rectangles for the buttons */
  for (i=0; i<11; i++)
    GP142_rectangleXY(YELLOW, W_X_START + i * W_BUT_WIDTH, W_Y_START, 
                      W_X_START + i * W_BUT_WIDTH + W_BUT_WIDTH, W_Y_START- W_BUT_HEIGHT, 1);

  /* draw the wind speed numbers */
  for (i=-5; i<=5; i++)
  {
    if (current_wind_speed == i)
      color_w_speed = SELECTED_COLOR;
    else
      color_w_speed =UNSELECTED_COLOR;
    GP142_printfXY(color_w_speed,i * W_BUT_WIDTH, W_Y_START-18 , 12, "%d", abs(i));
  }

  /* draw arrows */
  GP142_printfXY(BLACK, -28, -GP142_YMAX+25, 12, "Wind Speed");
  GP142_lineXY(BLACK, -32, -GP142_YMAX+31, -60, -GP142_YMAX+31,2);
  GP142_lineXY(BLACK, 32, -GP142_YMAX+31, 60, -GP142_YMAX+31,2);
  GP142_triangleXY(BLACK,-70, -GP142_YMAX+31, -60, -GP142_YMAX+35, -60, -GP142_YMAX+28,FILL);
  GP142_triangleXY(BLACK,70, -GP142_YMAX+31, 60, -GP142_YMAX+35, 60, -GP142_YMAX+28,FILL);
}

/* returns wind speed button number pressed */
void handle_wind_button(int *current_wind_speed, int x)
{
  *current_wind_speed = (x + (-1*(W_X_START)))/W_BUT_WIDTH - 5;
}


int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX-TOP_BUTTON_MARGIN) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT - TOP_BUTTON_MARGIN)))
    return BUTTON;
  else
    if ((mouse_x > W_X_START) && (mouse_x < (W_X_START + NUM_W_BUT * W_BUT_WIDTH))
        && (mouse_y < W_Y_START) && (mouse_y > (W_Y_START - W_BUT_HEIGHT)))
      return WIND_BUTTON;
  else if (mouse_y <= TOP_OF_DIRT)
    return DIRT;
  else
    return NONE;
}


void DrawLandscape(int sun_x, int sun_y, int is_day)
{  
int text_offset = -GP142_XMAX + 15, font_size = 12;


  /* Draw the sky first */
  if (is_day)
  GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		    GP142_XMAX, GP142_YMAX, FILL);
  else
  GP142_rectangleXY(BLACK, -GP142_XMAX, -GP142_YMAX,
		    GP142_XMAX, GP142_YMAX, FILL);
 
  /* Draw the sun or moon */
  if (is_day)
    GP142_circleXY(YELLOW, sun_x, sun_y , 20);
  else
    GP142_circleXY(WHITE, sun_x, sun_y , 20);   

  /* Draw the dirt */
  /* The dirt is a % of the height of the screen */
  GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
		    GP142_XMAX, TOP_OF_DIRT, FILL);
  GP142_printfXY (BLACK, text_offset, -GP142_YMAX+10, font_size,
	  "It's raining cats and dogs so click a weather type and a color, then click on the picture.");

  
  /*Draw the mountains */
  GP142_triangleXY(MED_GRAY,
		   -GP142_XMAX, TOP_OF_DIRT,
		   GP142_XMAX, TOP_OF_DIRT,
		   0, (4 * GP142_YMAX)/5 ,
		   FILL);

  GP142_triangleXY(OLIVE,
		   -( 5 * GP142_XMAX/4), TOP_OF_DIRT,
		   (GP142_XMAX)/3  , TOP_OF_DIRT,
		   -(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		   FILL);

  GP142_triangleXY(FOREST_GREEN,
		   -GP142_XMAX / 3 , TOP_OF_DIRT,
		   GP142_XMAX, TOP_OF_DIRT,
		   GP142_XMAX, (2 * GP142_YMAX)/3 ,
		   FILL);

  /* draw the river  */
  GP142_rectangleXY(BLUE,
      -GP142_XMAX , -GP142_YMAX+70,
      GP142_XMAX, -GP142_YMAX+100,
      FILL);

} /* end DrawLandscape */


/* draw the fish in the river */
void DrawFish(int fish_x_start)
{
int i;

  /*draw the fish  */        
  for (i=-GP142_XMAX + fish_x_start; i<(GP142_XMAX); i=i + 50)
  {
    GP142_triangleXY(SEA_GREEN,
		     i,    -GP142_YMAX+85 ,
		     i-10, -GP142_YMAX+90 ,
		     i-10, -GP142_YMAX+80 ,
		     FILL);
    GP142_ovalXY(SEA_GREEN,
         i,    -GP142_YMAX+80 ,
         i+20, -GP142_YMAX+90 ,
         FILL); 
  }
} 


/*  draw the flower */
void DrawFlower(int flower_age, int x, int y)
{
  int stem_length = flower_age;
  int flower_size = 0;

/* need to grow the flower slowly stem for first 14 cylces
   then center grows slowly, and leaves then petals  */

  if (flower_age > 14) stem_length = 14;

  GP142_lineXY(GREEN, x, y, x, y + stem_length, 1);

  if (flower_age > 10) {
    GP142_ovalXY(GREEN, x, y+4, x-4, y+6, FILL);
    GP142_ovalXY(GREEN, x, y+4, x+4, y+6, FILL);
  }
  if ((flower_age > 14) && (flower_age < 18))
    flower_size = flower_age-14;
  else if (flower_age >= 18) 
    flower_size = 3;
  GP142_circleXY(WHITE, x, y + 13 + flower_size, flower_size);

  flower_size = flower_age-18;
  if (flower_size > 2) flower_size = 2;

  if (flower_age>18) {
    GP142_circleXY(YELLOW, x-4, y+13, flower_size);
    GP142_circleXY(YELLOW, x+4, y+13, flower_size);
    GP142_circleXY(YELLOW, x-4, y+17, flower_size);
    GP142_circleXY(YELLOW, x+4, y+17, flower_size);
    GP142_circleXY(YELLOW, x,   y+19, flower_size);
    GP142_circleXY(YELLOW, x,   y+10, flower_size);
  }
}


void clean_up_precip_array(precip_struc all_precips[], int *precip_count)
{
  int i,j;
  for (i=0; i < *precip_count; i++)
  {
    if (all_precips[i].precip_type == NONE)
    {
      *precip_count=*precip_count - 1;
      for (j=i; j < *precip_count; j++)     
        all_precips[j] = all_precips[j+1];
    }
  }

} /* clean_up_precip_array */


void process_precipitation(precip_struc all_precips[], int *precip_count, int current_wind_speed)
{
  int i;
  for (i = 0; i < *precip_count; i++)
  {
    if (abs(all_precips[i].x) > (GP142_XMAX))
      all_precips[i].precip_type = NONE;
    switch (all_precips[i].precip_type)
    {
      case  RAIN  :
      case  SNOW  :
        all_precips[i].y-= 3;
        all_precips[i].x+= current_wind_speed;
        if (all_precips[i].y < TOP_OF_DIRT)
        {
          all_precips[i].precip_type = FLOWER;
          all_precips[i].age = 0;
        }
        break;
 
      case  FLOWER :
        all_precips[i].age++;    
        if (all_precips[i].age > 50)
          all_precips[i].precip_type = NONE;
        break;
      case CAT:
        if (all_precips[i].y > (TOP_OF_DIRT-20))
          {
            all_precips[i].y -= 4;
            all_precips[i].x += current_wind_speed;
          }
        else
          all_precips[i].x -= 3;
        break;
      case DOG:
        if (all_precips[i].y > (TOP_OF_DIRT-20))
          {
            all_precips[i].y -= 4;
            all_precips[i].x += current_wind_speed;
          }        
        else
          all_precips[i].x += 3;
        break;
    }
  }
  clean_up_precip_array(all_precips, precip_count);

} /* end process_precipitation */


/* Figure out which button was clicked */
void handle_button(int x, int y, int *current_precip, int *current_color,
		   int *clear_weather)
{
  int max_speed = 80;
  int buttonNum = 0;

  *clear_weather = FALSE; /* Assume we don't replant */

  /* If we're outside of this area, we could not have clicked on
   * a button.
   */
  if (x > -GP142_XMAX + BUTTON_WIDTH)  return;

  /* We know the mouse clicked in the column of buttons. */
  buttonNum = ((GP142_YMAX-TOP_BUTTON_MARGIN) - y)/BUTTON_HEIGHT;	 

  switch(buttonNum) {
  case 0:
    *current_precip = RAIN;
    break;
  case 1:
    *current_precip = SNOW;
    break;
  case 2:
    *current_precip = CAT;
    break;
  case 3:
    *current_precip = DOG;
    break;
  case 4:
    *current_color = WHITE;
    break;
  case 5:
    *current_color = BLUE;
    break;
  case 6:
    *current_color = GREEN;  
    break;
  case 7:
    *clear_weather = TRUE;
    break; /* Signal to clear up the weather */
  default:
		/*Bad news if this is ever reached! */
		GP142_printfXY (RED, 50, 50, 14, "BAAADD NEWS!");
    break;
  }

}


/* Draws the buttons */
void DrawButtons(int current_precip, int current_color)
{
  int i, text_offset = -GP142_XMAX + 15, font_size = 12;

  /* draw rectangles for each button */
  for(i = 0; i < NUM_BUTTONS; i++) {
    GP142_rectangleXY(YELLOW,
		      -GP142_XMAX + 4,
		      GP142_YMAX - i * BUTTON_HEIGHT - 2 - TOP_BUTTON_MARGIN,
		      -GP142_XMAX + BUTTON_WIDTH,
		      GP142_YMAX - (i + 1)*BUTTON_HEIGHT - 2 - TOP_BUTTON_MARGIN, 1);
  }

  /* display button text */
  /* "selected" buttons are highlighted (i.e., have a different color)*/
  /* This whole mess could be cleaned and put into a function
  if we could use strings. */
  i = GP142_YMAX - 25 - TOP_BUTTON_MARGIN;
  if (current_precip == RAIN)
    GP142_printfXY(SELECTED_COLOR, text_offset, i, font_size, "Rain");	
  else
    GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "Rain");	

  i -= BUTTON_HEIGHT;
  if (current_precip == SNOW)
    GP142_printfXY(SELECTED_COLOR, text_offset, i, font_size, "Snow");
  else
    GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "Snow");

  i -= BUTTON_HEIGHT;
  if (current_precip == CAT)
    GP142_printfXY(SELECTED_COLOR, text_offset, i, font_size, "Cat");
  else
    GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "Cat");

  i -= BUTTON_HEIGHT;
  if (current_precip == DOG)
    GP142_printfXY(SELECTED_COLOR, text_offset, i, font_size, "Dog");
  else
    GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "Dog");

 
  /* Buttons for Colors */
  i -= BUTTON_HEIGHT;
  if (current_color == WHITE)
    GP142_printfXY(SELECTED_COLOR, text_offset, i, font_size, "White");	
  else
    GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "White");

  i -= BUTTON_HEIGHT;
  if (current_color == BLUE)
    GP142_printfXY(SELECTED_COLOR, text_offset, i, font_size, "Blue");
  else
    GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "Blue");

  i -= BUTTON_HEIGHT;
  if (current_color == GREEN)
    GP142_printfXY(SELECTED_COLOR, text_offset, i, font_size,"Green");
  else
    GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "Green");    
 

  /* This button is never "selected" */
  i -= BUTTON_HEIGHT;
  GP142_printfXY(UNSELECTED_COLOR, text_offset, i, font_size, "Cease!");

}
